﻿/*
*	FILE			: ControlFrame.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DATE            : March 20, 2019
*	DESCRIPTION 	: Used for authentication and sensor control
*/

using BBSensorsErrorLog;
using System;
using System.Windows.Forms;

namespace MiBand_Heartrate
{
    public partial class ControlFrame : Form
    {
        private MiBand device;
        public static bool isRunning = false;

        public ControlFrame()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Load the selected MiBand
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ControlFrame_Load(object sender, EventArgs e)
        {
            device = new MiBand(DeviceControl.Instance.BluetoothManager);
        }

        /// <summary>
        /// Authentication button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void authButton_Click(object sender, EventArgs e)
        {
            device.Authenticate(OnAuth);
        }

        private delegate void OnAuthHandler(MiBand d, bool s);

        /// <summary>
        /// Authenticate MiBand
        /// </summary>
        /// <param name="d"></param>
        /// <param name="s"></param>
        private void OnAuth(MiBand d, bool s)
        {
            if (InvokeRequired)
            {
                OnAuthHandler c = new OnAuthHandler(OnAuth);
                Invoke(c, new object[] { d, s });
            }
            else
            {
                if (s)
                {
                    connectionStatusLabel.Text = "Connected | Auth";
                    startButton.Enabled = true;
                    isRunning = true;
                }
                else
                {
                    isRunning = false;
                    MessageBox.Show("Auth failed !", "Device error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    BBSensorErrorLog.SaveErrorData("Device error - Auth failed !");
                }
            }
        }

        /// <summary>
        /// Start Data collection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            startButton.Enabled = false;
            device.ContinuousMode = true;
            device.StartMonitorHeartrate();
            device.HeartrateChanged += DeviceControl.Instance.WriteRealtimeHeartrateInFile;
            device.HeartrateChanged += OnHeartrateChange;
            stopButton.Enabled = true;
            isRunning = true;
        }

        /// <summary>
        /// Stop Data Collection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            stopButton.Enabled = false;
            device.StopMonitorHeartrate();
            device.HeartrateChanged -= OnHeartrateChange;
            heartrateLabel.Text = "--";
            startButton.Enabled = true;
        }

        private delegate void OnHeartrateChangeHandler(MiBand d, ushort v);

        /// <summary>
        /// Display heart rate change
        /// </summary>
        /// <param name="d"></param>
        /// <param name="v"></param>
        private void OnHeartrateChange(MiBand d, ushort v)
        {
            if (InvokeRequired)
            {
                OnHeartrateChangeHandler c = new OnHeartrateChangeHandler(OnHeartrateChange);
                Invoke(c, new object[] { d, v });
            }
            else
            {
                heartrateLabel.Text = string.Format("{0} bpm", v);
            }
        }

        /// <summary>
        /// Disconnect when application it closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ControlFrame_FormClosing(object sender, FormClosingEventArgs e)
        {
            device.StopMonitorHeartrate();
            if (DeviceControl.Instance.BluetoothManager.HasDeviceConnected())
            {
                DeviceControl.Instance.BluetoothManager.Disconnect();
            }
        }
    }
}