﻿/*
*	FILE			: BBSD2DB.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DATE            : March 20, 2019
*	DESCRIPTION 	: This application decrypt and insert data into SQL Server database.
*/

using BBSensorDataEncryptor;
using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Windows.Forms;

namespace BBPSensorData2DataBase
{
    public partial class BBSD2DB : Form
    {
        #region Variables

        private static string line;
        private static string[] BBPSensorParsedData;
        private static DataTable BBPresenseSensorsData = new DataTable("dbo.BBPresenseSensorsData");
        private static DataTable BBThermostatData = new DataTable("dbo.BBThermostatData");
        private static SqlConnection cn;
        private static string tableType;

        public delegate void UpdateProgressSub(int CurrentLine);

        private bool abortXmlProcess;
        private int FileIndex = 0;
        private string OutputFolder;
        private int linesWritten = 0;
        private static int linePerFile = 100;
        private static int maxFileCount = 100;

        /// <summary>
        /// Initializing Connection parameters
        /// </summary>
        private static string DB_Connection = ConfigurationManager.ConnectionStrings["BB_db_CONNECTION"].ConnectionString;

        private long Results;

        /// <summary>
        /// Variables for bulk insert
        /// </summary>
        private int blockSize = 0;

        #endregion Variables

        #region Constructor

        public BBSD2DB()
        {
            InitializeComponent();
        }

        #endregion Constructor

        #region CreateTables

        /// <summary>
        /// Setting up data table for presence sensor
        /// </summary>
        private static void CreatePresenseSensorTable()
        {
            BBPresenseSensorsData.Columns.Add("pLocation", typeof(string));
            BBPresenseSensorsData.Columns.Add("pStatus", typeof(string));
            BBPresenseSensorsData.Columns.Add("pTemperature", typeof(float));
            BBPresenseSensorsData.Columns.Add("pHumidity", typeof(float));
            BBPresenseSensorsData.Columns.Add("pTime", typeof(string));
            BBPresenseSensorsData.Columns.Add("pDate", typeof(DateTime));
            BBPresenseSensorsData.Columns.Add("pDateTimeStamp", typeof(DateTime));
        }

        /// <summary>
        /// Setting up data table for thermostat
        /// </summary>
        private static void CreateThermostatTable()
        {
            BBThermostatData.Columns.Add("tHumidity", typeof(float));
            BBThermostatData.Columns.Add("tTemperature", typeof(float));
            BBThermostatData.Columns.Add("tT_heat", typeof(float));
            BBThermostatData.Columns.Add("tT_cool", typeof(float));
            BBThermostatData.Columns.Add("tTmode", typeof(string));
            BBThermostatData.Columns.Add("tFmode", typeof(string));
            BBThermostatData.Columns.Add("tHold", typeof(string));
            BBThermostatData.Columns.Add("tTstate", typeof(string));
            BBThermostatData.Columns.Add("tTtarget", typeof(string));
            BBThermostatData.Columns.Add("tProgramMode", typeof(string));
            BBThermostatData.Columns.Add("tFstate", typeof(string));
            BBThermostatData.Columns.Add("tTime", typeof(string));
            BBThermostatData.Columns.Add("tDate", typeof(DateTime));
            BBThermostatData.Columns.Add("tDateTimeStamp", typeof(string));
        }

        #endregion CreateTables

        #region TableInsert

        /// <summary>
        /// parse sensor's data
        /// </summary>
        /// <param name="BBData"></param>
        private static void BB_Database(string BBData)
        {
            BBPSensorParsedData = BBData.Split(',');

            if (tableType == "BBPSD")
            {
                /// Getting Sensor Data parsed from string
                try
                {
                    BBPresenseSensorsData.Rows.Add(BBPSensorParsedData[0],
                        BBPSensorParsedData[1],
                        BBPSensorParsedData[2],
                        BBPSensorParsedData[3],
                        BBPSensorParsedData[4],
                        BBPSensorParsedData[5],
                        BBPSensorParsedData[5] + " " + BBPSensorParsedData[4]);
                    BBPresenseSensorsData.AcceptChanges();
                }
                catch (Exception)
                {
                    //MessageBox.Show("Error inserting into data table", "Data Table Insert", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            if (tableType == "BBTD")
            {
                /// Getting Sensor Data parsed from string
                try
                {
                    BBThermostatData.Rows.Add(BBPSensorParsedData[0],
                        BBPSensorParsedData[1],
                        BBPSensorParsedData[2],
                        BBPSensorParsedData[3],
                        BBPSensorParsedData[4],
                        BBPSensorParsedData[5],
                        BBPSensorParsedData[6],
                        BBPSensorParsedData[7],
                        BBPSensorParsedData[8],
                        BBPSensorParsedData[9],
                        BBPSensorParsedData[10],
                        BBPSensorParsedData[11],
                        BBPSensorParsedData[12],
                        BBPSensorParsedData[12] + " " + BBPSensorParsedData[11]);
                    BBThermostatData.AcceptChanges();
                }
                catch (Exception)
                {
                    //MessageBox.Show("Error inserting into data table", "Data Table Insert", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        #endregion TableInsert

        #region DataBaseInsert

        /// <summary>
        /// Insert data into database
        /// </summary>
        private void insertData()
        {
            // Presence Sensor table
            if (tableType == "BBPSD")
            {
                try
                {
                    cn = new SqlConnection(DB_Connection);
                    //Open connection to do bulk insert
                    cn.Open();
                    DBConnectionStatus.Image = Properties.Resources.trafficlight_green;
                    DBTransferStatus.Image = Properties.Resources.trafficlight_green;

                    using (SqlBulkCopy bulkInsert = new SqlBulkCopy(cn))
                    {
                        // Destination and source table have to have the same name and column name
                        // and data type.
                        bulkInsert.DestinationTableName = BBPresenseSensorsData.TableName;

                        // The layout for the bulk insert is DataTable Column name , DataBase Column name
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pLocation", "pLocation"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pStatus", "pStatus"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pTemperature", "pTemperature"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pHumidity", "pHumidity"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pTime", "pTime"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pDate", "pDate"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("pDateTimeStamp", "pDateTimeStamp"));
                        bulkInsert.WriteToServer(BBPresenseSensorsData);
                        bulkInsert.Close();
                    }

                    //If successful the DataTable is cleared and is ready for new data
                    BBPresenseSensorsData.Rows.Clear();

                    //Close connection after insertion
                    cn.Close();
                }
                catch (Exception ex)
                {
                    pIcon.Image = Properties.Resources.error;
                    DBConnectionStatus.Image = Properties.Resources.trafficlight_red;
                    DBTransferStatus.Image = Properties.Resources.trafficlight_red;
                    MessageBox.Show(ex.ToString());
                }
            }
            // Thermostat Table
            if (tableType == "BBTD")
            {
                DBConnectionStatus.Image = Properties.Resources.trafficlight_green;
                try
                {
                    cn = new SqlConnection(DB_Connection);
                    //Open connection to do bulk insert
                    cn.Open();

                    DBTransferStatus.Image = Properties.Resources.trafficlight_green;

                    using (SqlBulkCopy bulkInsert = new SqlBulkCopy(cn))
                    {
                        // Destination and source table have to have the same name and column name
                        // and data type.
                        bulkInsert.DestinationTableName = BBThermostatData.TableName;

                        // The layout for the bulk insert is DataTable Column name , DataBase Column name
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tHumidity", "tHumidity"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tTemperature", "tTemperature"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tT_heat", "tT_heat"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tT_cool", "tT_cool"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tTmode", "tTmode"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tFmode", "tFmode"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tHold", "tHold"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tTstate", "tTstate"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tTtarget", "tTtarget"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tProgramMode", "tProgramMode"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tFstate", "tFstate"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tTime", "tTime"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tDate", "tDate"));
                        bulkInsert.ColumnMappings.Add(new SqlBulkCopyColumnMapping("tDateTimeStamp", "tDateTimeStamp"));
                        bulkInsert.WriteToServer(BBThermostatData);
                        bulkInsert.Close();
                    }

                    //If successful the DataTable is cleared and is ready for new data
                    BBThermostatData.Rows.Clear();

                    //Close connection after insertion
                    cn.Close();
                }
                catch
                {
                    DBConnectionStatus.Image = Properties.Resources.trafficlight_red;
                    DBTransferStatus.Image = Properties.Resources.trafficlight_red;
                    pIcon.Image = Properties.Resources.error;
                    MessageBox.Show("Error inserting into database", "Database Insert", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        #endregion DataBaseInsert

        #region GetFileSize

        /// <summary>
        /// Calculating File size
        /// </summary>
        /// <param name="TotalSize"></param>
        /// <returns></returns>
        private static string CalculateFileSize(long TotalSize)
        {
            if (TotalSize >= 1073741824)
            {
                decimal FileSize = decimal.Divide(TotalSize, 1073741824);
                string msg = string.Format("{0:##.##} GB", FileSize);
                linePerFile = 1000;
                MessageBox.Show("Application may show Not Responding due to file size \n " + "Current File Size: " + msg + ". During operation \n you may open log folder to check operation progress", "XML2SQL", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return string.Format("{0:##.##} GB", FileSize);
            }
            else if (TotalSize >= 1048576 && TotalSize < 1073741824)
            {
                decimal FileSize = decimal.Divide(TotalSize, 1048576);
                return string.Format("{0:##.##} MB", FileSize);
            }
            else if (TotalSize >= 1024 && TotalSize < 1048576)
            {
                decimal FileSize = decimal.Divide(TotalSize, 1024);
                //maxFileCount = 1;
                //linePerFile = 10;
                return string.Format("{0:##.##} KB", FileSize);
            }
            else
            {
                return "0 Bytes";
            }
        }

        #endregion GetFileSize

        /// <summary>
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BBPSD2DB_Load(object sender, EventArgs e)
        {
            CreatePresenseSensorTable();
            CreateThermostatTable();
        }

        #region BrowseFile

        /// <summary>
        /// Executes the open file command
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void browseFileBtn_Click(object sender, EventArgs e)
        {
            pIcon.Image = Properties.Resources.loading__2016_10_29_16_24_24_UTC_;
            DBConnectionStatus.Image = Properties.Resources.trafficlight_yellow;
            DBTransferStatus.Image = Properties.Resources.trafficlight_yellow;
            xmlProgress.Value = 0;
            pPerc.Text = Convert.ToString((xmlProgress.Value / xmlProgress.Maximum) * 100) + "%";

            OpenFileDialog openXML = new OpenFileDialog();

            if (openXML.ShowDialog() == DialogResult.OK)
            {
                xmlFile.Text = openXML.FileName;
            }

            ///Getting file info
            FileInfo F_Info = new FileInfo(xmlFile.Text);
            Results = F_Info.Length;

            //Calculation file size and displaying file info
            fSize.Text = CalculateFileSize(Results);
            fName.Text = F_Info.Name;

            // Determine which table to insert data
            if (F_Info.Name.Contains("thermostat"))
            {
                tableType = "BBTD";
            }
            if (F_Info.Name.Contains("presence"))
            {
                tableType = "BBPSD";
            }
        }

        #endregion BrowseFile

        #region ReadFile

        /// <summary>
        /// Stream the encrypted file and decrypt it. If file too large, file is split
        /// </summary>
        /// <param name="FilePath"></param>
        /// <param name="LineCount"></param>
        /// <param name="MaxXlmFiles"></param>
        /// <param name="Status"></param>
        /// <param name="IsAbort"></param>
        public void SplitXML(string FilePath, int LineCount, int MaxXlmFiles, UpdateProgressSub Status, ref bool IsAbort)
        {
            // Open the XML file for reading
            StreamReader Reader = new StreamReader(FilePath);

            // Create the output directory
            OutputFolder = FilePath + "_Log";
            if (Directory.Exists(FilePath) == false)
            {
                Directory.CreateDirectory(OutputFolder);
            }

            //Initializing progress bar parameters
            xmlProgress.Maximum = MaxXlmFiles;
            xmlProgress.Step = 1;

            blockSize = 0;

            while ((line = Reader.ReadLine()) != null)
            {
                //Update progress bar
                xmlProgress.PerformStep();

                pPerc.Text = Convert.ToString((xmlProgress.Value / xmlProgress.Maximum) * 100) + "%";

                blockSize++;

                // Update progress
                FileIndex += 1;
                if (Status != null)
                {
                    Status.Invoke((FileIndex - 1) * LineCount);
                }

                line = Reader.ReadLine();

                //XLM extraction
                BB_Database(BBSensorDataDecryption.DecryptData(line));

                blockSize++;

                if (blockSize == 1000)
                {
                    blockSize = 0;
                    insertData();
                }
            }

            Reader.Close();
            insertData();
        }

        #endregion ReadFile

        /// <summary>
        /// Calls the splitXml method and notify the user of progress success or failure
        /// </summary>
        public void SplitIt()
        {
            startBtn.Enabled = false;
            cancelBtn.Enabled = true;

            try
            {
                SplitXML(xmlFile.Text, linePerFile, maxFileCount, UpdateProgress, ref abortXmlProcess);

                if (!abortXmlProcess)
                {
                    pIcon.Image = Properties.Resources.pass;
                }
                else
                {
                    abortXmlProcess = false;
                    MessageBox.Show("Process Canceled Successfully", "Extraction Process", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception)
            {
                //MessageBox.Show(ex.Message, "Splitting Process", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                startBtn.Enabled = true;
                cancelBtn.Enabled = false;
            }
        }

        /// <summary>
        /// Display the total lines written after split
        /// </summary>
        /// <param name="CurrentLine"></param>
        public void UpdateProgress(int CurrentLine)
        {
            linesWritten = CurrentLine;
        }

        /// <summary>
        /// Cancel the process
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cancelBtn_Click(object sender, EventArgs e)
        {
            abortXmlProcess = true;
        }

        /// <summary>
        /// Start the process
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startBtn_Click(object sender, EventArgs e)
        {
            SplitIt();
        }
    }
}