﻿/*
*	FILE			: BBSensorConnectionManager.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DATE            : March 20, 2019
*	DESCRIPTION 	: This file manages the connection for the Presence sensor.
*/

using BBSensorsErrorLog;
using System;
using System.Net;
using System.Net.Sockets;

namespace BBSensorConnection
{
    public class BBSensorConnectionManager
    {
        #region Variables

        private static int counter;
        private static string sensorIP;
        private static short sensorPort;

        #endregion Variables

        #region Connection Properties

        public static IPAddress LocalIPAddress => IPAddress.Parse(sensorIP);
        public static int Port => sensorPort;
        public static IPEndPoint EndPoint => new IPEndPoint(LocalIPAddress, Port);

        #endregion Connection Properties

        #region Connection String

        /// <summary>
        /// Parse connection data
        /// </summary>
        /// <param name="connection"></param>
        public static void parseConnectionData(string connection)
        {
            counter = 0;

            string[] connectionData = connection.Split(',');
            foreach (string connectionDataDigits in connectionData)
            {
                if (counter == 0) { sensorIP = connectionDataDigits; }
                if (counter == 1) { sensorPort = Convert.ToInt16(connectionDataDigits); }

                counter++;
            }
        }

        #endregion Connection String

        #region Create Listener and Socket

        /// <summary>
        /// Create the connection needed for the TCP connection
        /// </summary>
        /// <returns></returns>
        public static Socket CreateListener()
        {
            Socket socket = null;
            try
            {
                // Create a TCP/IP socket.
                socket = CreateSocket();
                socket.Bind(EndPoint);
                socket.Listen(10);
            }
            catch (Exception ex)
            {
                BBSensorErrorLog.SaveErrorData("PIR Sensor " + ex.Message.ToString());
            }

            return socket;
        }

        public static Socket CreateSocket()
        {
            return new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
        }

        #endregion Create Listener and Socket
    }
}