﻿/*
*	FILE			: BBthermostatDataRequest.cs
*	PROJECT			: Black Box
*	PROGRAMMER		: BRIAN HINDS
*	FIRST VERSION	: 1.0.0.0
*	DESCRIPTION 	: This library file manage the data request to the thermostat.
*/

using BBSensorsErrorLog;
using BBTaskerThermostatStatus;
using Newtonsoft.Json;
using System;

namespace BBThermostatDataEncryptor
{
    public class BBThermostatDataRequest
    {
        private static string BBThermostatData;
        private static string BBThermostatAllStat;
        private static string BBThermostatHumidity;

        /// <summary>
        /// This method request and parse the thermostat stat data
        /// </summary>
        private static void GetBBThermostatAllData()
        {
            BBThermostatAllStat = BBThermostatConnection.BBThermostatConnect.BBThermostatRequestA();
            BBThermostatDefinitions.BBThermostatDefinition buffer = JsonConvert.DeserializeObject<BBThermostatDefinitions.BBThermostatDefinition>(BBThermostatAllStat);

            // Current temperature
            BBThermostatData += TempConvertor(buffer.temp) + ",";

            // Temporary target heat set point
            BBThermostatData += TempConvertor(buffer.t_heat) + ",";

            // Temporary target cool set point
            BBThermostatData += TempConvertor(buffer.t_cool) + ",";

            //Thermostat operation mode
            if (buffer.tmode.ToString() == "0")
            {
                BBThermostatData += "OFF,";
            }
            if (buffer.tmode.ToString() == "1")
            {
                BBThermostatData += "HEAT,";
            }
            if (buffer.tmode.ToString() == "2")
            {
                BBThermostatData += "COOL,";
            }
            if (buffer.tmode.ToString() == "3")
            {
                BBThermostatData += "AUTO,";
            }

            //Thermostat fan operation mode
            if (buffer.fmode.ToString() == "0")
            {
                BBThermostatData += "AUTO,";
            }
            if (buffer.fmode.ToString() == "1")
            {
                BBThermostatData += "AUTO/CIRCULATE,";
            }
            if (buffer.fmode.ToString() == "2")
            {
                BBThermostatData += "ON,";
            }

            //Thermostat target temperature hold status
            if (buffer.hold.ToString() == "0")
            {
                BBThermostatData += "DISABLE,";
            }
            if (buffer.hold.ToString() == "1")
            {
                BBThermostatData += "ENABLE,";
            }

            //Thermostat HVAC Operation state
            if (buffer.tstate.ToString() == "0")
            {
                BBThermostatData += "OFF,";
            }
            if (buffer.tstate.ToString() == "1")
            {
                BBThermostatData += "HEAT,";
            }
            if (buffer.tstate.ToString() == "2")
            {
                BBThermostatData += "COOL,";
            }

            //Thermostat current Operation target
            if (buffer.ttarget.ToString() == "0")
            {
                BBThermostatData += "OFF,";
            }
            if (buffer.ttarget.ToString() == "1")
            {
                BBThermostatData += "HEAT,";
            }
            if (buffer.ttarget.ToString() == "2")
            {
                BBThermostatData += "COOL,";
            }

            //Thermostat operation mode
            if (buffer.program_mode.ToString() == "0")
            {
                BBThermostatData += "Program A,";
            }
            if (buffer.program_mode.ToString() == "1")
            {
                BBThermostatData += "Program B,";
            }
            if (buffer.program_mode.ToString() == "2")
            {
                BBThermostatData += "Vacation,";
            }
            if (buffer.program_mode.ToString() == "3")
            {
                BBThermostatData += "Holiday,";
            }

            //Thermostat fan operation status
            if (buffer.fstate.ToString() == "0")
            {
                BBThermostatData += "OFF,";
            }
            if (buffer.fstate.ToString() == "1")
            {
                BBThermostatData += "ON,";
            }

            // Current date and time the data was recorded
            BBThermostatData += DateTime.Now.ToString("HH:mm:ss") + "," + DateTime.Now.ToString("yyyy-MM-dd");
        }

        /// <summary>
        /// This method request and parse the thermostat humidity data
        /// </summary>
        private static void GetBBThermostatHumidtyData()
        {
            BBThermostatHumidity = BBThermostatConnection.BBThermostatConnect.BBThermostatRequestB();
            BBThermostatDefinitions.BBThermostatDefinition buffer = JsonConvert.DeserializeObject<BBThermostatDefinitions.BBThermostatDefinition>(BBThermostatHumidity);
            BBThermostatData += buffer.humidity.ToString() + ",";
        }

        /// <summary>
        /// This method converts the temperature from imperial to metric
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        private static string TempConvertor(float value)
        {
            return Convert.ToString(((int)value - 32) * 5 / 9);
        }

        /// <summary>
        /// Get thermostat data and encrypt
        /// </summary>
        public static void BBThermosataCheck()
        {
            try
            {
                GetBBThermostatHumidtyData();
                GetBBThermostatAllData();
                BBThermostatDataEncryption.EncryptData(BBThermostatData);
                Console.WriteLine(BBThermostatData);
                Console.WriteLine(Environment.NewLine);
                BBThermostatData = "";
            }
            catch (Exception ex)
            {
                // Set operation status to false
                BBThermostatStatus.SensorStatus(false);
                BBSensorErrorLog.SaveErrorData(ex.Message.ToString());
            }
        }
    }
}